@@ -1,10 +1,10 @@
 package Net::DNS::RR::AFSDB;
 
 #
-# $Id: AFSDB.pm 1382 2015-08-26 20:39:33Z willem $
+# $Id: AFSDB.pm 1406 2015-10-05 08:25:49Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1382 $)[1];
+$VERSION = (qw$LastChangedRevision: 1406 $)[1];
 
 
 use strict;
@@ -35,17 +35,16 @@ sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 	my ( $offset, @opaque ) = @_;
 
-	return '' unless $self->{hostname};
-	my $rdata = pack 'n', $self->subtype;
-	$rdata .= $self->{hostname}->encode( $offset + length($rdata), @opaque );
+	my $hostname = $self->{hostname} || return '';
+	pack 'n a*', $self->subtype, $hostname->encode( $offset + 2, @opaque );
 }
 
 
 sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return '' unless $self->{hostname};
-	join ' ', $self->subtype, $self->{hostname}->string;
+	my $hostname = $self->{hostname} || return '';
+	join ' ', $self->subtype, $hostname->string;
 }
 
 
@@ -69,7 +68,7 @@ sub hostname {
 	my $self = shift;
 
 	$self->{hostname} = new Net::DNS::DomainName2535(shift) if scalar @_;
-	$self->{hostname}->name if defined wantarray && $self->{hostname};
+	$self->{hostname}->name if $self->{hostname};
 }
 
 
