@@ -1,10 +1,10 @@
 package Net::DNS::Resolver::Recurse;
 
 #
-# $Id: Recurse.pm 1406 2015-10-05 08:25:49Z willem $
+# $Id: Recurse.pm 1415 2015-10-13 07:24:58Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1406 $)[1];
+$VERSION = (qw$LastChangedRevision: 1415 $)[1];
 
 
 =head1 NAME
@@ -86,51 +86,39 @@ sub send {
 	return &Net::DNS::Resolver::Base::send if ref $_[1];	# send Net::DNS::Packet
 
 	my $self = shift;
-	my $res	 = bless {cache => {}, %$self}, ref($self);	# Note: cache discarded after query
+	my $res = bless {cache => {'.' => $root}, %$self}, ref($self);
 
 	my $question = new Net::DNS::Question(@_);
 	my $original = pop(@_);					# sneaky extra argument needed
 	$original = $question unless ref($original);		# to preserve original request
 
 	my ( $head, @tail ) = $question->{qname}->label;
-	unless ( defined $head ) {
-		return $root if $root;				# root servers cached indefinitely
-
-		my $defres = new Net::DNS::Resolver( retry => 1 );
-		my @config = $defres->nameservers( $res->hints );
-		$defres->nameservers( @config, $res->_hints );	# hints, resolv.conf, _hints
-		$defres->udppacketsize(1280);
-
-		my $packet = $defres->send( '.', 'NS' );
-		my %glue;
-
-		# uncoverable branch false	# depends on external data
-		if ($packet) {
-			my @rr = $packet->answer, $packet->authority;
-			my @auth = grep $_->type eq 'NS', @rr;
-			my %auth = map { lc $_->nsdname => 1 } @auth;
-			my @glue = grep $auth{lc $_->name}, $packet->additional;
-			foreach ( grep $_->can('address'), @glue ) {
-				push @{$glue{lc $_->name}}, $_->address;
-			}
-		}
-
-		$defres->nameservers( $res->_hints );
-		$defres->nameservers( map @$_, values %glue );
-		$defres->recurse(0);
-		return $root = $defres->send( '.', 'NS' );	# authoritative server
-	}
-
 	my $domain = lc join( '.', @tail ) || '.';
 	my $nslist = $res->{cache}->{$domain} ||= [];
+	unless ( defined $head ) {
+		$root = $nslist;				# root servers cached indefinitely
+
+		my $defres = new Net::DNS::Resolver();
+		my @config = $defres->nameservers( $res->hints );
+		my $packet = $defres->send( '.', 'NS' );
+
+		# uncoverable branch false			# repeat using authoritative server
+		my @auth = grep $_->type eq 'NS', $packet->answer, $packet->authority if $packet;
+		$defres->nameservers( map $_->nsdname, @auth );
+		$defres->recurse(0);
+		$packet = $defres->send( '.', 'NS' );
+
+		$defres->nameservers( $res->_hints );		# fall back on internal hints 
+		$packet = $defres->send( '.', 'NS' ) unless $packet;	# uncoverable branch true
+		return $packet;
+	}
+
 	if ( scalar @$nslist ) {
 		$self->_diag("using cached nameservers for $domain");
 	} else {
 		$domain = lc $question->qname if $question->qtype ne 'NULL';
 		my $packet = $res->send( $domain, 'NULL', 'IN', $original );
-
-		# uncoverable branch true	# depends on external data
-		return unless $packet;
+		return unless $packet;				# uncoverable branch true
 
 		my @answer = $packet->answer;			# return authoritative answer
 		return $packet if $packet->header->aa && grep $_->name eq $original->qname, @answer;
@@ -150,49 +138,30 @@ sub send {
 			@{$res->{cache}->{$zone}} = @list;
 			return $packet if length($zone) > length($domain);
 			$self->_diag("cache nameservers for $zone");
-			push @$nslist, @list;
+			@$nslist = @list;
 		}
 	}
 
+
 	my $query = new Net::DNS::Packet();
 	$query->push( question => $original );
 	$res->recurse(0);
 
-	my @a = grep ref($_), @$nslist;
-	splice @a, 0, 0, splice( @a, int( rand scalar @a ) );	# cut deck
+	splice @$nslist, 0, 0, splice( @$nslist, int( rand scalar @$nslist ) );	   # cut deck
 
-	foreach (@a) {
-		my @ns = $res->nameservers(@$_);
+	foreach my $ns (@$nslist) {
+		unless ( ref $ns ) {
+			$self->_diag("find missing glue for $ns");
+			$ns = [$res->nameservers($ns)];		# substitute IP list in situ
+		}
 
-		# uncoverable branch true	# depends on external data
-		next unless scalar @ns;
+		my @ns = $res->nameservers(@$ns);
+		next unless scalar @ns;				# uncoverable branch true
 
 		my $reply = $res->send($query);
+		next unless $reply;				# uncoverable branch true
 
-		# uncoverable branch true	# depends on external data
-		next unless $reply;
-
-		# uncoverable branch true	# depends on external data
-		next unless $reply->header->rcode eq 'NOERROR';
-
-		$self->_callback($reply);
-		return $reply;
-	}
-
-	foreach my $ns ( grep !ref($_), @$nslist ) {
-		$self->_diag("find missing glue for $ns");
-		$ns = [$res->nameservers($ns)];			# substitute IP list in situ
-
-		# uncoverable branch true	# depends on external data
-		next unless scalar @$ns;
-
-		my $reply = $res->send($query);
-
-		# uncoverable branch true	# depends on external data
-		next unless $reply;
-
-		# uncoverable branch true	# depends on external data
-		next unless $reply->header->rcode eq 'NOERROR';
+		next unless $reply->header->rcode eq 'NOERROR'; # uncoverable branch true
 
 		$self->_callback($reply);
 		return $reply;
